@@ -1,13 +1,20 @@
-/* Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -47,7 +54,7 @@
 using std::min;
 using std::max;
 
-/* max size of the log message */
+/* max size of log messages (error log, plugins' logging, general log) */
 #define MAX_LOG_BUFFER_SIZE 1024
 #define MAX_TIME_SIZE 32
 
@@ -960,11 +967,14 @@ bool LOGGER::flush_logs(THD *thd)
 
 /**
   Close and reopen the slow log (with locks).
-  
-  @returns FALSE.
+
+  @return Error status code
+    @retval false Ok
+    @retval true  Error
 */
 bool LOGGER::flush_slow_log()
 {
+  bool result= false;
   /*
     Now we lock logger, as nobody should be able to use logging routines while
     log tables are closed
@@ -973,22 +983,25 @@ bool LOGGER::flush_slow_log()
 
   /* Reopen slow log file */
   if (opt_slow_log)
-    file_log_handler->get_mysql_slow_log()->reopen_file();
+    result= file_log_handler->get_mysql_slow_log()->reopen_file();
 
   /* End of log flush */
   logger.unlock();
 
-  return 0;
+  return result;
 }
 
 
 /**
   Close and reopen the general log (with locks).
 
-  @returns FALSE.
+  @return Error status code
+    @retval false Ok
+    @retval true  Error
 */
 bool LOGGER::flush_general_log()
 {
+  bool result= false;
   /*
     Now we lock logger, as nobody should be able to use logging routines while
     log tables are closed
@@ -997,12 +1010,12 @@ bool LOGGER::flush_general_log()
 
   /* Reopen general log file */
   if (opt_log)
-    file_log_handler->get_mysql_log()->reopen_file();
+    result= file_log_handler->get_mysql_log()->reopen_file();
 
   /* End of log flush */
   logger.unlock();
 
-  return 0;
+  return result;
 }
 
 
@@ -1054,7 +1067,7 @@ bool LOGGER::slow_log_print(THD *thd, const char *query, uint query_length)
 
     /* fill in user_host value: the format is "%s[%s] @ %s [%s]" */
     user_host_len= (strxnmov(user_host_buff, MAX_USER_HOST_SIZE,
-                             sctx->priv_user ? sctx->priv_user : "", "[",
+                             sctx->priv_user, "[",
                              sctx->user ? sctx->user : "", "] @ ",
                              sctx->get_host()->length() ?
                              sctx->get_host()->ptr() : "", " [",
@@ -1114,12 +1127,6 @@ bool LOGGER::general_log_write(THD *thd, enum enum_server_command command,
 
   current_time= my_time(0);
 
-  mysql_audit_general_log(thd, current_time,
-                          user_host_buff, user_host_len,
-                          command_name[(uint) command].str,
-                          command_name[(uint) command].length,
-                          query, query_length);
-                        
   while (*current_handler)
     error|= (*current_handler++)->
       log_general(thd, current_time, user_host_buff,
@@ -1416,7 +1423,7 @@ static int find_uniq_filename(char *name)
   file_info= dir_info->dir_entry;
   for (i= dir_info->number_off_files ; i-- ; file_info++)
   {
-    if (memcmp(file_info->name, start, length) == 0 &&
+    if (strncmp(file_info->name, start, length) == 0 &&
 	test_if_number(file_info->name+length, &number,0))
     {
       set_if_bigger(max_found,(ulong) number);
@@ -1500,6 +1507,78 @@ bool MYSQL_LOG::init_and_set_log_file_name(const char *log_name,
 }
 
 
+bool is_valid_log_name(const char *name, size_t len)
+{
+  if (len > 3)
+  {
+    const char *tail= name + len - 4;
+    if (my_strcasecmp(system_charset_info, tail, ".ini") == 0 ||
+        my_strcasecmp(system_charset_info, tail, ".cnf") == 0)
+    {
+      return false;
+    }
+  }
+  return true;
+}
+
+
+/**
+  Get the real log file name, and possibly reopen file.
+
+  Use realpath() to get the path with symbolic links
+  expanded. Then, close the file, and reopen the real path using the
+  O_NOFOLLOW flag. This will reject following symbolic links.
+
+  @param          file                  File descriptor.
+  @param          log_file_key          Key for P_S instrumentation.
+  @param          open_flags            Flags to use for opening the file.
+  @param          opened_file_name      Name of the open fd.
+
+  @retval file descriptor to open file with 'real_file_name', or '-1'
+          in case of errors.
+*/
+
+#ifndef _WIN32
+static File mysql_file_real_name_reopen(File file,
+#ifdef HAVE_PSI_INTERFACE
+                                        PSI_file_key log_file_key,
+#endif
+                                        int open_flags,
+                                        const char *opened_file_name)
+{
+  DBUG_ASSERT(file);
+  DBUG_ASSERT(opened_file_name);
+
+  /* Buffer for realpath must have capacity for PATH_MAX. */
+  char real_file_name[PATH_MAX];
+
+  /* Get realpath, validate, open realpath with O_NOFOLLOW. */
+  if (realpath(opened_file_name, real_file_name) == NULL)
+  {
+    (void) mysql_file_close(file, MYF(0));
+    return -1;
+  }
+
+  if (mysql_file_close(file, MYF(0)))
+    return -1;
+
+  if (strlen(real_file_name) > FN_REFLEN)
+    return -1;
+
+  if (!is_valid_log_name(real_file_name, strlen(real_file_name)))
+  {
+    sql_print_error("Invalid log file name after expanding symlinks: '%s'",
+                    real_file_name);
+    return -1;
+  }
+
+  return mysql_file_open(log_file_key, real_file_name,
+                         open_flags | O_NOFOLLOW,
+                         MYF(MY_WME | ME_WAITTANG));
+}
+#endif // _WIN32
+
+
 /*
   Open a (new) log file.
 
@@ -1570,6 +1649,18 @@ bool MYSQL_LOG::open(
                              MYF(MY_WME | ME_WAITTANG))) < 0)
     goto err;
 
+#ifndef _WIN32
+  /* Reopen and validate path. */
+  if ((log_type_arg == LOG_UNKNOWN || log_type_arg == LOG_NORMAL) &&
+      (file= mysql_file_real_name_reopen(file,
+#ifdef HAVE_PSI_INTERFACE
+                                         log_file_key,
+#endif
+                                         open_flags,
+                                         log_file_name)) < 0)
+    goto err;
+#endif // _WIN32
+
   if ((pos= mysql_file_tell(file, MYF(MY_WME))) == MY_FILEPOS_ERROR)
   {
     if (my_errno == ESPIPE)
@@ -1611,20 +1702,11 @@ bool MYSQL_LOG::open(
   DBUG_RETURN(0);
 
 err:
-  if (log_type == LOG_BIN && binlogging_impossible_mode == ABORT_SERVER)
+  if (log_type == LOG_BIN && binlog_error_action == ABORT_SERVER)
   {
-    THD *thd= current_thd;
-    /*
-      On fatal error when code enters here we should forcefully clear the
-      previous errors so that a new critical error message can be pushed
-      to the client side.
-     */
-    thd->clear_error();
-    my_error(ER_BINLOG_LOGGING_IMPOSSIBLE, MYF(0), "Either disk is full or "
-             "file system is read only while opening the binlog. Aborting the "
-             "server");
-    thd->protocol->end_statement();
-    _exit(EXIT_FAILURE);
+    exec_binlog_error_action_abort("Either disk is full or file system is read "
+                                   "only while opening the binlog. Aborting the"
+                                   " server.");
   }
   else
     sql_print_error("Could not open %s for logging (error %d). "
@@ -1745,27 +1827,27 @@ int MYSQL_LOG::generate_new_name(char *new_name, const char *log_name)
 }
 
 
-/*
-  Reopen the log file
+/**
+  Reopen the log file.
 
-  SYNOPSIS
-    reopen_file()
-
-  DESCRIPTION
-    Reopen the log file. The method is used during FLUSH LOGS
+  @note  Reopen the log file. The method is used during FLUSH LOGS
     and locks LOCK_log mutex
+
+  @return Error status code
+    @retval false Ok
+    @retval true  Error
 */
 
-
-void MYSQL_QUERY_LOG::reopen_file()
+bool MYSQL_QUERY_LOG::reopen_file()
 {
   char *save_name;
+  bool result= false;
 
   DBUG_ENTER("MYSQL_LOG::reopen_file");
   if (!is_open())
   {
     DBUG_PRINT("info",("log is closed"));
-    DBUG_VOID_RETURN;
+    DBUG_RETURN(result);
   }
 
   mysql_mutex_lock(&LOCK_log);
@@ -1778,16 +1860,16 @@ void MYSQL_QUERY_LOG::reopen_file()
      Note that at this point, log_state != LOG_CLOSED (important for is_open()).
   */
 
-  open(
+  result= open(
 #ifdef HAVE_PSI_INTERFACE
-       m_log_file_key,
+      m_log_file_key,
 #endif
-       save_name, log_type, 0, io_cache_type);
+      save_name, log_type, 0, io_cache_type);
   my_free(save_name);
 
   mysql_mutex_unlock(&LOCK_log);
 
-  DBUG_VOID_RETURN;
+  DBUG_RETURN(result);
 }
 
 
@@ -2089,7 +2171,8 @@ bool slow_log_print(THD *thd, const char *query, uint query_length)
 }
 
 
-bool LOGGER::log_command(THD *thd, enum enum_server_command command)
+bool LOGGER::log_command(THD *thd, enum enum_server_command command,
+                         const char *query_str, size_t query_length)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   Security_context *sctx= thd->security_ctx;
@@ -2098,6 +2181,12 @@ bool LOGGER::log_command(THD *thd, enum enum_server_command command)
     Log command if we have at least one log event handler enabled and want
     to log this king of commands
   */
+
+  /* Audit notification when no general log handler present */
+  mysql_audit_general_log(thd, command_name[(uint) command].str,
+                          command_name[(uint) command].length,
+                          query_str, query_length);
+
   if (*general_log_handler_list && (what_to_log & (1L << (uint) command)))
   {
     if ((thd->variables.option_bits & OPTION_LOG_OFF)
@@ -2124,7 +2213,7 @@ bool general_log_print(THD *thd, enum enum_server_command command,
   uint error= 0;
 
   /* Print the message to the buffer if we want to log this king of commands */
-  if (! logger.log_command(thd, command))
+  if (! logger.log_command(thd, command, "", 0))
     return FALSE;
 
   va_start(args, format);
@@ -2138,7 +2227,7 @@ bool general_log_write(THD *thd, enum enum_server_command command,
                        const char *query, uint query_length)
 {
   /* Write the message to the log if we want to log this king of commands */
-  if (logger.log_command(thd, command))
+  if (logger.log_command(thd, command, query, query_length))
     return logger.general_log_write(thd, command, query, query_length);
 
   return FALSE;
@@ -2349,7 +2438,7 @@ static void print_buffer_to_file(enum loglevel level, const char *buffer,
 */
 int vprint_msg_to_log(enum loglevel level, const char *format, va_list args)
 {
-  char   buff[1024];
+  char   buff[MAX_LOG_BUFFER_SIZE];
   size_t length;
   DBUG_ENTER("vprint_msg_to_log");
 
@@ -2408,7 +2497,7 @@ extern "C"
 int my_plugin_log_message(MYSQL_PLUGIN *plugin_ptr, plugin_log_level level,
                           const char *format, ...)
 {
-  char format2[MYSQL_ERRMSG_SIZE];
+  char format2[MAX_LOG_BUFFER_SIZE];
   int ret;
   loglevel lvl;
   struct st_plugin_int *plugin = static_cast<st_plugin_int *> (*plugin_ptr);
@@ -2955,3 +3044,54 @@ int TC_LOG::using_heuristic_recover()
   sql_print_information("Please restart mysqld without --tc-heuristic-recover");
   return 1;
 }
+
+/**
+  When a fatal error occurs due to which binary logging becomes impossible and
+  the user specified binlog_error_action= ABORT_SERVER the following function is
+  invoked. This function pushes the appropriate error message to client and logs
+  the same to server error log and then aborts the server.
+
+  @param err_string          Error string which specifies the exact error
+                             message from the caller.
+
+  @retval
+    none
+*/
+void exec_binlog_error_action_abort(const char* err_string)
+{
+  THD *thd= current_thd;
+  /*
+    When the code enters here it means that there was an error at higher layer
+    and my_error function could have been invoked to let the client know what
+    went wrong during the execution.
+
+    But these errors will not let the client know that the server is going to
+    abort. Even if we add an additional my_error function call at this point
+    client will be able to see only the first error message that was set
+    during the very first invocation of my_error function call.
+
+    The advantage of having multiple my_error function calls are visible when
+    the server is up and running and user issues SHOW WARNINGS or SHOW ERROR
+    calls. In this special scenario server will be immediately aborted and
+    user will not be able execute the above SHOW commands.
+
+    Hence we clear the previous errors and push one critical error message to
+    clients.
+   */
+  if (thd)
+  {
+    if (thd->is_error())
+      thd->clear_error();
+    /*
+      Adding ME_NOREFRESH flag will ensure that the error is sent to both
+      client and to the server error log as well.
+     */
+    my_error(ER_BINLOG_LOGGING_IMPOSSIBLE, MYF(ME_NOREFRESH + ME_FATALERROR),
+             err_string);
+    thd->protocol->end_statement();
+  }
+  else
+    sql_print_error("%s",err_string);
+  abort();
+}
+
